@@ -1,5 +1,6 @@
 package Test::Weaken;
 
+use 5.006;
 use strict;
 use warnings;
 
@@ -7,16 +8,12 @@ require Exporter;
 
 use base qw(Exporter);
 our @EXPORT_OK = qw(leaks poof);
-our $VERSION   = '3.004000';
+our $VERSION   = '3.008000';
 
 # use Smart::Comments;
 
 ### <where> Using Smart Comments ...
 
-## no critic (BuiltinFunctions::ProhibitStringyEval)
-$VERSION = eval $VERSION;
-## use critic
-
 =begin Implementation:
 
 The basic strategy: get a list of all the objects which allocate memory,
@@ -129,10 +126,18 @@ sub follow {
                 last FIND_CHILDREN;
             } ## end if ( $object_type eq 'HASH' )
 
-            # GLOB and LVALUE are not tracked by default,
+            # GLOB is not tracked by default,
+            # but we follow ties
+            if ( $object_type eq 'GLOB' ) {
+                if ( my $tied_var = tied *${$follow_probe} ) {
+                    push @child_probes, \($tied_var);
+                }
+                last FIND_CHILDREN;
+            } ## end if ( $object_type eq 'GLOB' )
+
+            # LVALUE is not tracked by default,
             # but we follow ties
             if (   $object_type eq 'SCALAR'
-                or $object_type eq 'GLOB'
                 or $object_type eq 'VSTRING'
                 or $object_type eq 'LVALUE' )
             {
@@ -140,7 +145,7 @@ sub follow {
                     push @child_probes, \($tied_var);
                 }
                 last FIND_CHILDREN;
-            } ## end if ( $object_type eq 'SCALAR' or $object_type eq 'GLOB'...)
+            } ## end if ( $object_type eq 'SCALAR' or $object_type eq ...)
 
             if ( $object_type eq 'REF' ) {
                 if ( my $tied_var = tied ${$follow_probe} ) {
@@ -469,7 +474,8 @@ sub Test::Weaken::check_ignore {
         my $array_context = wantarray;
 
         my $before_weak =
-            ( ref $probe_ref eq 'REF' and Scalar::Util::isweak( ${$probe_ref} ) );
+            ( ref $probe_ref eq 'REF'
+                and Scalar::Util::isweak( ${$probe_ref} ) );
         my $before_dump =
             Data::Dumper->new( [$probe_ref], [qw(proberef)] )
             ->Maxdepth($compare_depth)->Dump();
@@ -496,7 +502,8 @@ sub Test::Weaken::check_ignore {
         }
 
         my $after_weak =
-            ( ref $probe_ref eq 'REF' and Scalar::Util::isweak( ${$probe_ref} ) );
+            ( ref $probe_ref eq 'REF'
+                and Scalar::Util::isweak( ${$probe_ref} ) );
         my $after_dump =
             Data::Dumper->new( [$probe_ref], [qw(proberef)] )
             ->Maxdepth($compare_depth)->Dump();
@@ -532,7 +539,7 @@ sub Test::Weaken::check_ignore {
 
             $error_count++;
 
-            my $message .= q{};
+            my $message = q{};
             $message .= $before_reporting_dump
                 if $include_before;
             $message .= $after_reporting_dump
@@ -558,6 +565,8 @@ sub Test::Weaken::check_ignore {
 
 __END__
 
+=for stopwords abend misdesign misimplement unfreed deallocated deallocation referenceable builtin recursing globals Builtin OO destructor VSTRING LVALUE unevaluated subdirectory refaddr refcount indiscernable XSUB Mortalizing mortalize mortalizing pre-calculated subr refcounts recurses dereferences filehandle Kegler perldoc AnnoCPAN CPAN CPAN's perl Ryde jettero Juerd morgon perrin Perlmonks
+
 =head1 NAME
 
 Test::Weaken - Test that freed memory objects were, indeed, freed
@@ -576,49 +585,38 @@ is_file($_, 't/synopsis.t', 'synopsis')
     use Data::Dumper;
     use Math::BigInt;
     use Math::BigFloat;
-    use Carp;
-    use English qw( -no_match_vars );
 
     my $good_test = sub {
         my $obj1 = Math::BigInt->new('42');
         my $obj2 = Math::BigFloat->new('7.11');
-        [ $obj1, $obj2 ];
+        return [ $obj1, $obj2 ];
     };
-
     if ( !leaks($good_test) ) {
-        print "No leaks in test 1\n"
-            or Carp::croak("Cannot print to STDOUT: $ERRNO");
-    }
-    else {
-        print "There were memory leaks from test 1!\n"
-            or Carp::croak("Cannot print to STDOUT: $ERRNO");
+        print "No leaks in test 1\n";
+    } else {
+        print "There were memory leaks from test 1!\n";
     }
 
     my $bad_test = sub {
-        my $array = [ 42, 711 ];
-        push @{$array}, $array;
-        $array;
+        my $arrayref = [ 42, 711 ];
+        push @{$arrayref}, $arrayref;  # circular reference
+        return $arrayref;
     };
-
     my $bad_destructor = sub {'I am useless'};
-
     my $tester = Test::Weaken::leaks(
         {   constructor => $bad_test,
             destructor  => $bad_destructor,
         }
     );
     if ($tester) {
-        my $unfreed_proberefs = $tester->unfreed_proberefs();
-        my $unfreed_count     = @{$unfreed_proberefs};
         printf "Test 2: %d of %d original references were not freed\n",
-            $tester->unfreed_count(), $tester->probe_count()
-            or Carp::croak("Cannot print to STDOUT: $ERRNO");
-        print "These are the probe references to the unfreed objects:\n"
-            or Carp::croak("Cannot print to STDOUT: $ERRNO");
+            $tester->unfreed_count(), $tester->probe_count();
+
+        my $unfreed_proberefs = $tester->unfreed_proberefs();
+        print "These are the probe references to the unfreed objects:\n";
         for my $ix ( 0 .. $#{$unfreed_proberefs} ) {
             print Data::Dumper->Dump( [ $unfreed_proberefs->[$ix] ],
-                ["unfreed_$ix"] )
-                or Carp::croak("Cannot print to STDOUT: $ERRNO");
+                ["unfreed_$ix"] );
         }
     }
 
@@ -672,7 +670,7 @@ L<Test::Weaken|/"NAME"> can deal with circular references without going
 into infinite loops.
 L<Test::Weaken|/"NAME"> will not visit the same Perl data object twice.
 
-=head2 Data Objects, Blessed Objects and Structures
+=head2 Data objects, blessed objects and structures
 
 B<Object> is a heavily overloaded term in the Perl world.
 This document will use the term B<Perl data object>
@@ -688,14 +686,23 @@ L<bless builtin|perlfunc/"bless">, will be called a B<blessed object>.
 In this document,
 a Perl B<data structure> (often just called a B<structure>)
 is any group of Perl objects that are
-B<co-mortal> -- expected to be destroyed at the same time.
+co-mortal.
+B<Co-mortal> means that the maintainer
+expects those objects to be destroyed at the same time.
+For example, if a group of Perl objects is referenced,
+directly or indirectly,
+through a hash,
+and is referenced only through that hash,
+a programmer will usually expect all of those objects
+to be destroyed when the hash is.
+
+Perl data structures can be any set of
+Perl data objects.
 Since the question is one of I<expected> lifetime,
 whether an object is part of a data structure
 is, in the last analysis, subjective.
-Perl data structures can be any set of
-Perl data objects.
 
-=head2 The Contents of a Data Structure
+=head2 The contents of a data structure
 
 A B<data structure> must have one object
 that is designated as its B<top object>.
@@ -712,7 +719,7 @@ The closure should return
 a reference to the test structure.
 This reference is called the B<test structure reference>.
 
-=head2 Children and Descendants
+=head2 Children and descendants
 
 The elements of an array are B<children> of the array.
 The values of a hash are B<children> of the hash.
@@ -732,10 +739,10 @@ then the second data object is an B<ancestor> of the first object.
 A data object is considered to be a descendant of itself,
 and also to be one of its own ancestors.
 
-L<Test::Weaken|/"NAME">'s default assumption,
+L<Test::Weaken|/"NAME">'s default assumption is
 that the contents of a data structure are the same as
-its descendants, works
-for many cases,
+its descendants.
+This works for many cases,
 but not for all.
 Ways to deal with
 descendants that are not contents,
@@ -747,7 +754,7 @@ such as inside-out objects,
 are dealt with in
 L<the section on nieces|/"Nieces">.
 
-=head2 Builtin Types
+=head2 Builtin types
 
 This document will refer to the builtin type of objects.
 Perl's B<builtin types> are the types Perl originally gives objects,
@@ -763,7 +770,7 @@ L<ref function|perlfunc/"ref"> returns the builtin type.
 The L<Scalar::Util/reftype function> always returns the builtin type,
 even for blessed objects.
 
-=head2 Persistent Objects
+=head2 Persistent objects
 
 As a practical matter, a descendant that is not
 part of the contents of a
@@ -825,10 +832,11 @@ the wrapper array itself, plus the contents
 of the lab rat.
 
 It is not always easy to find the right objects to put into the wrapper array.
-In particular, determining the contents of the lab rat may
-require what
-amounts to a recursive scan of the descendants of the lab rat's
+For example, determining the contents of the lab rat may
+require a recursive scan from the lab rat's
 top object.
+Depending on the logical structure of the lab rat,
+this may be far from trivial.
 
 As an alternative to using a wrapper,
 it is possible to have L<Test::Weaken|/"NAME"> add
@@ -836,7 +844,7 @@ contents "on the fly," while it is scanning the lab rat.
 This can be done using L<the C<contents> named argument|/contents>,
 which takes a closure as its value.
 
-=head2 Why the Test Structure is Passed via a Closure
+=head2 Why the test structure is passed via a closure
 
 L<Test::Weaken|/"NAME"> gets its test structure reference
 indirectly,
@@ -877,7 +885,7 @@ A test structure constructor
 can return a reference to a test structure
 created from Perl data objects in any scope the user desires.
 
-=head2 Returns and Exceptions
+=head2 Returns and exceptions
 
 The methods of L<Test::Weaken|/"NAME"> do not return errors.
 Errors are always thrown as exceptions.
@@ -1436,7 +1444,7 @@ the count returned by L</"unfreed_count">.
 
 =head1 ADVANCED TECHNIQUES
 
-=head2 Tracing Leaks
+=head2 Tracing leaks
 
 =head3 Avoidance
 
@@ -1455,7 +1463,7 @@ the last successful test.
 Often, examining these changes is enough to
 tell where the leak was introduced.
 
-=head3 Adding Tags
+=head3 Adding tags
 
 The L</"unfreed_proberefs"> method returns an array containing
 probes to
@@ -1467,7 +1475,7 @@ If circumstances allow it,
 you might find it useful to add "tag" elements to arrays and hashes
 to aid in identifying the source of a leak.
 
-=head3 Using Referent Addresses
+=head3 Using referent addresses
 
 You can quasi-uniquely identify data objects using
 the referent addresses of the probe references.
@@ -1487,14 +1495,14 @@ happily, it is
 the referent address that both zero addition
 and L<refaddr|Scalar::Util/refaddr> return.
 
-=head3 Other Techniques
+=head3 Other techniques
 
 Sometimes, when you are interested in why an object is not being freed,
 you want to seek out the reference
 that keeps the object's refcount above 0.
 L<Devel::FindRef> can be useful for this.
 
-=head2 More about Quasi-unique Addresses
+=head2 More about quasi-unique addresses
 
 I call referent addresses "quasi-unique", because they are only
 unique at a
@@ -1524,7 +1532,7 @@ object created at the same address.
 But for most practical programming purposes,
 two indiscernable data objects can be regarded as the same object.
 
-=head2 Debugging Ignore Subroutines
+=head2 Debugging ignore subroutines
 
 =head3 check_ignore
 
@@ -1676,6 +1684,97 @@ The hacked version can reside anywhere,
 and does not need to
 be part of the L<Test::Weaken|/"NAME"> package.
 
+=head1 XSUB Mortalizing
+
+When a C language XSUB returns a newly created scalar it should "mortalize"
+so the scalar is freed once the caller has finished with it (see
+L<perlguts/Reference Counts and Mortality>).  Failing to do so leaks memory.
+
+    SV *ret = newSViv(123);
+    sv_2mortal (ret);   /* must mortalize */
+    XPUSHs (ret);
+
+C<Test::Weaken> can check this by taking a reference to the returned
+scalar,
+
+    my $leaks = leaks (sub {
+                         return \( somexsub() );
+                       });
+    if ($leaks) ...
+
+Don't store to a local scalar and then return that.  Doing so will only
+check the local scalar, not the one returned by C<somexsub()>.
+
+If you want the value for further calculations then take a reference to the
+return then look through that for the value.
+
+    leaks (sub {
+             my $ref = \( somexsub() );
+             my $value = $$ref;
+             # ... do something with $value
+             return $ref;
+           });
+
+If an XSUB returns a list of values then take a reference to each as
+follows.  This works because C<map> and C<for> make the loop variable
+(either C<$_> or named) an alias to each value successively.
+
+    leaks (sub {
+             return [ map {\$_} somexsub() ];
+           });
+
+    # or with a for loop
+    leaks (sub {
+             my @refs;
+             foreach my $value (somexsub()) {
+               push @refs, \$value;
+             }
+             return \@refs;
+           });
+
+Don't store a returned list to an array (either named or anonymous) as this
+copies into new scalars in that array and the returned ones from
+C<somexsub()> then aren't checked.
+
+If you want the values from a list for extra calculations then take the
+references first and look through them for the values like the single case
+above.  For example,
+
+    leaks (sub {
+             my @refs = map {\$_} somexsub();
+             my $first_ref = $refs[0]
+             my $value = $$first_ref;
+             # ... do something with $value
+             return \@refs;
+           });
+
+An XSUB might deliberately return the same scalar each time, perhaps a
+pre-calculated constant or a global variable it maintains.  In that case the
+scalar intentionally won't weaken away and this C<leaks()> checking is not
+applicable.
+
+Returning the same scalar every time occurs in pure Perl too from an
+anonymous constant subr, such as created by the C<constant> module (see
+L<constant>).  This is unlikely to arise directly, but could be encountered
+through a scalar ref within an object etc.
+
+    # FOO() returns same scalar every time
+    *FOO = sub () { 123 };
+
+    # likewise from the constant module
+    use constant BAR => 456;
+
+There's no way to tell the intended lifespan of an XSUB return, but
+generally if the code has any sort of C<newSV()> or C<sv_newmortal()> etc
+making a new scalar every time then it ought to weaken away.
+
+The details of an XSUB return are often hidden in a F<typemap> file for
+brevity and consistency (see L<perlxs/The Typemap>).  The supplied types
+(F<Extutils/typemap>) are hard to get wrong, but code with explicit
+C<PUSHs()> etc is worth checking.  Generally too much mortalizing causes
+negative refcounts and probable segfaults, and not enough mortalizing leaks
+memory.
+
 =head1 EXPORTS
 
 By default, L<Test::Weaken|/"NAME"> exports nothing.
@@ -1712,7 +1811,7 @@ the value of that probe reference will be C<undef>.
 If a probe reference is still defined at this point,
 it refers to an unfreed Perl data object.
 
-=head2 Tracked Objects
+=head2 Tracked objects
 
 By default,
 objects of builtin types ARRAY, HASH, REF,
@@ -1757,7 +1856,7 @@ Any builtin type may be added to the list of builtin types to be
 tracked with the
 L<tracked_types named argument|/"tracked_types">.
 
-=head2 Examining Objects for Children
+=head2 Examining objects for children
 
 Objects of builtin type
 ARRAY, HASH, REF,
@@ -1777,6 +1876,20 @@ can be seen as a limitation, because
 closures do hold internal references to data objects.
 Future versions of L<Test::Weaken|/"NAME"> may examine CODE objects.
 
+A variable of builtin type GLOB may be
+a scalar which was assigned a GLOB value
+(a scalar-GLOB) or it may simply be a GLOB (a pure-GLOB).
+The issue that arises from
+L<Test::Weaken|/"NAME">'s standpoint is that,
+in the case of a scalar-GLOB,
+the scalar and the GLOB may be tied separately.
+At present,
+the underlying tied variable of the scalar side of a
+scalar-GLOB is ignored.
+Only the underlying tied variable of the GLOB
+is a child for
+L<Test::Weaken|/"NAME">'s purposes.
+
 The default method of recursing through a test structure
 to find its contents can be customized.
 The L</ignore> callback can be used to force an object
@@ -1832,13 +1945,29 @@ L<http://search.cpan.org/dist/Test-Weaken>
 
 =head1 SEE ALSO
 
-Potential users will want to compare L<Test::Memory::Cycle> and
-L<Devel::Cycle>, which examine
-existing data structures non-destructively.
-L<Devel::Leak> also covers similar ground, although it requires
-Perl to be compiled with C<-DDEBUGGING> in order to work.  L<Devel::Cycle>
-looks inside closures if PadWalker is present, a feature L<Test::Weaken|/"NAME">
-does not have at present.
+L<Test::Weaken|/"NAME">
+at this point is robust
+and has 
+seen extensive use.
+Its tracking of memory is careful enough
+that it has even stumbled upon
+L<a bug in perl
+itself|http://rt.perl.org/rt3/Public/Bug/Display.html?id=67838>.
+
+L<Test::Weaken::Gtk2>
+is a CPAN Module of "helper" functions
+for L<Test::Weaken|/"NAME">.
+L<Test::Weaken::Gtk2>
+is specifically aimed at the needs of users
+of L<Gtk2>,
+but can also be used
+as an example of how
+an expert user extends and adapts
+L<Test::Weaken|/"NAME">.
+Kevin Ryde, the author of 
+L<Test::Weaken::Gtk2>,
+has been a important contributor to 
+L<Test::Weaken|/"NAME">.
 
 =head1 ACKNOWLEDGEMENTS
 
@@ -1853,7 +1982,7 @@ For version 3.000000, Kevin also provided patches.
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2007-2009 Jeffrey Kegler, all rights reserved.
+Copyright 2012 Jeffrey Kegler, all rights reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the same terms as Perl 5.10.
@@ -1862,9 +1991,10 @@ it under the same terms as Perl 5.10.
 
 1;    # End of Test::Weaken
 
+#   fill-column: 100
+#
 # Local Variables:
 #   mode: cperl
 #   cperl-indent-level: 4
-#   fill-column: 100
 # End:
 # vim: expandtab shiftwidth=4:
